// SVTriangleMesh.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_TRIANGLES_ADJACENCY_EXT
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLES
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=24

#version 120
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform float shadowLength;

varying in vec3 positionIn[];
varying in vec4 colorIn[];

varying out vec4 color;

vec3 CastedPosS(vec3 pos){
    vec3 lightVec = gl_LightSource[0].position.xyz - pos * gl_LightSource[0].position[3];
    vec3 nlightVec = -normalize(lightVec);
    return pos + nlightVec * shadowLength * 0.0003;
}

vec3 CastedPosL(vec3 pos){
    vec3 lightVec = gl_LightSource[0].position.xyz - pos * gl_LightSource[0].position[3];
    vec3 nlightVec = -normalize(lightVec);
    return pos + nlightVec * shadowLength;
}

void main(void){

    vec3 lightVec = gl_LightSource[0].position.xyz - positionIn[0] * gl_LightSource[0].position[3];
    vec3 nlightVec = -normalize(lightVec);
    
    vec3 polyNormal = cross(vec3(positionIn[2] - positionIn[0]), vec3(positionIn[4] - positionIn[0]));
    float litOrNot = dot(lightVec, polyNormal);
    
    if (litOrNot < 0) return;
    
    vec3 polyNormalABdash = cross(vec3(positionIn[1] - positionIn[0]), vec3(positionIn[2] - positionIn[0]));
    float litOrNotABdash = dot(lightVec, polyNormalABdash);


    vec3 lightVec2 = gl_LightSource[0].position.xyz - positionIn[2] * gl_LightSource[0].position[3];
    vec3 polyNormalBCdash = cross(vec3(positionIn[3] - positionIn[2]), vec3(positionIn[4] - positionIn[2]));
    float litOrNotBCdash = dot(lightVec2, polyNormalBCdash);

    vec3 lightVec4 = gl_LightSource[0].position.xyz - positionIn[4] * gl_LightSource[0].position[3];
    vec3 polyNormalCAdash = cross(vec3(positionIn[5] - positionIn[4]), vec3(positionIn[0] - positionIn[4]));
    float litOrNotCAdash = dot(lightVec4, polyNormalCAdash);

    vec3 offsetS = nlightVec * shadowLength * 0.0003;
    //vec3 offsetL = nlightVec * shadowLength;

    color = colorIn[0];

    gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[0]), 1);
    EmitVertex();
    gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[2]), 1);
    EmitVertex();
    gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[4]), 1);
    EmitVertex();
    EndPrimitive();

    if (litOrNotABdash < 0){
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[0]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[2]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[2]), 1);
	EmitVertex();
	EndPrimitive();
	
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[0]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[0]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[2]), 1);
	EmitVertex();
	EndPrimitive();
    }

    if (litOrNotBCdash < 0){
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[2]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[4]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[4]), 1);
	EmitVertex();
	EndPrimitive();

	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[2]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[2]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[4]), 1);
	EmitVertex();
	EndPrimitive();
    }

    if (litOrNotCAdash < 0){
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[4]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[0]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[0]), 1);
	EmitVertex();
	EndPrimitive();
	
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosS(positionIn[4]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[4]), 1);
	EmitVertex();
	gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[0]), 1);
	EmitVertex();
	EndPrimitive();
    }

    gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[0]), 1);
    EmitVertex();
    gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[4]), 1);
    EmitVertex();
    gl_Position = gl_ProjectionMatrix * vec4(CastedPosL(positionIn[2]), 1);
    EmitVertex();
    EndPrimitive();
}
